/*
-----------------------------------------------------------
 Copyright (c) Youssef Hmamouche - All Rights Reserved.

 This software is subject to the General Public License.
 ----------------------------------------------------------
*/

#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <glib.h>

#include "lkcp.h"
#include "dbg.h"
#include "entries.h"
#include "scr_err_hndl.h"
#include "commit_script.h"

extern WINDOW* bgrdwin;
extern WINDOW* lkcpwin;
extern WINDOW* menuwin;
extern WINDOW* viewwin;
extern WINDOW* modewin;
extern WINDOW* helpwin;
extern WINDOW* fieldwin;

extern void draw_helpwin();

#ifdef DEBUG
extern FILE* f;
#endif

FORM* entryform;
FIELD* entryfield[2];

extern int lkcp_mode;
extern char* banner;

/* return codes:
	0: no permission to edit or view item
	1: entry saved;
	2: user hit ESC. do not save;
*/

int max_col, max_row;

/* 	I'm not fond of functions (other than main) that span beyond one screen worth
	of text, but this one had to or at least until "somebody" steps up and rewrites it. 
*/
int editentry(const char* name){
	int key=0, fd=0;
	int count=0;
	char* name_value = (char*) malloc(100+1);
	char* abs_path_name = (char*) malloc(100+1);
	char* current_dir_name = (char*) get_current_dir_name();
	LOG_CALL(0);
        if(abs_path_name ==NULL || name_value ==NULL || current_dir_name == NULL){
			LOG_CALL(1);
                        return -1;
	}

	if(strlen(name) <=0){
		LOG_CALL(1);
                return -1;
	}

	memset(name_value, '\0', 100+1);
	memset(abs_path_name, '\0', 100+1);

	getmaxyx(stdscr,max_row, max_col);

	if(max_row < 13 || max_col < 95){
		sfree(name_value);
		sfree(abs_path_name);
		nonfatal_error_screen(stdscr);
		LOG_CALL(1);
		return 0; /* this implies: goto mainwhile in lkcp_main.c*/
	}	
	helpwin = subwin(stdscr, 13, 95, 20, 0);

	if(helpwin == NULL){
		(void) wprintw(stdscr, "Error creating helpwin");
		(void) refresh();
		sfree(name_value);
		sfree(abs_path_name);
		sfree(current_dir_name);
		LOG_CALL(1);
		return 0;
	}
	strncpy(abs_path_name, current_dir_name, 100);
	strcat(abs_path_name, "/");
        strncat(abs_path_name, name, strlen(name));
	
	draw_helpwin(helpwin, name);
	
	if(lkcp_mode == VIEW_MODE){
		fd = open(abs_path_name, O_RDONLY|O_NONBLOCK);	
		if(fd < 0){
			if( errno == EACCES){
				strcpy(name_value, "permission denied");
				count = strlen(name_value);
			}
			else {
				strcpy(name_value, "unknown error");
				count = strlen(name_value);
			}		
		}
		else if((count=read(fd, name_value, 100)) < 0){
			(void) printw("Error reading from %s", abs_path_name);
			(void) close(fd);
			LOG_CALL(1);
			return 0;
		}
		
		(void) close(fd);
	}
	else {	/* EDIT MODE */
		fd = open(abs_path_name, O_RDWR|O_NONBLOCK);
		if(fd < 0){
			if((fd = open(abs_path_name, O_RDONLY|O_NONBLOCK)) <0){
                        	(void) wprintw(stdscr, "Error opening %s", abs_path_name);
				(void) close(fd);
				LOG_CALL(1);
				return 0;
			}
			else {
				(void) wmove(stdscr, max_row-10, 1);
				(void) wattron(stdscr, A_UNDERLINE);
				(void) wattron(stdscr, COLOR_PAIR(1));
				(void) wprintw(stdscr, "field %s is a read-only file", abs_path_name);
				(void) wattroff(stdscr, A_UNDERLINE);
				(void) wattroff(stdscr, COLOR_PAIR(1));
				(void) refresh();
			}
			if((count=read(fd, name_value, 100))<0){
				(void) (void) wprintw(stdscr, "Error opening %s", abs_path_name);
				(void) close(fd);
				LOG_CALL(1);
				return 0;
			} 
		}
		else if((count=read(fd, name_value, 100)) < 0){
                        (void) printw("Error reading from %s", abs_path_name);
			(void) close(fd);
			LOG_CALL(1);
			return 0;
                }
		(void) close(fd);


	}
	stripoff(name_value); /* get rid of \t \n from values */
	
	if(lkcp_mode == VIEW_MODE){
		int maxy=0, maxx=0;
		getmaxyx(stdscr, maxy, maxx);	
		if( maxy < 3 || maxx < (strlen(abs_path_name)+count+5)){ /* such an error is not fatal */
			sfree(name_value);
			sfree(abs_path_name);
			nonfatal_error_screen(stdscr);
			LOG_CALL(1);
			return 0;
		}
		/* Warning: if you change values here(viewwin), make sure you change them above. As 
		 * long as this is relative to LKCP menuwin, it's OK (the check is done earlier).
		 */
		viewwin = subwin(stdscr, 3, (strlen(banner)+strlen(LKCP_VERSION)+53), 15, 0);
		if(viewwin == NULL){
                	(void) wprintw(stdscr, "Error creating helpwin");
                	(void) refresh();
			LOG_CALL(1);
                	return 0;
        	}

        	draw_entrywin(viewwin, abs_path_name, name_value, count);

		while((key=getch()) !=ESC);
		(void) werase(viewwin);
		(void) werase(modewin);
		(void) werase(helpwin);
		(void) touchwin(stdscr);	
	 	sfree(name_value);
		sfree(abs_path_name);
		LOG_CALL(1);
		return 1;
	}
	else { 	/* EDIT MODE */
		int key=0, maxy=0, maxx=0;
		char* hold_user_val = (char*) malloc(100+1);

		if(hold_user_val == NULL){
			sfree(name_value);
			sfree(abs_path_name);
			LOG_CALL(1);
			return -1; /* ENOMEM*/
		}

		memset(hold_user_val, '\0', 100+1);

		getmaxyx(stdscr, maxy, maxx);
		if(maxy < 3 || maxx < (strlen(abs_path_name)+count+5)){
			sfree(name_value);
			sfree(abs_path_name);
			nonfatal_error_screen(stdscr);
			LOG_CALL(1);
			return 0; /* goto mainwhile in lkcp_main.c */
		}
		viewwin = subwin(stdscr, 3, (strlen(banner)+strlen(LKCP_VERSION)+53), 15, 0);

		if(viewwin == NULL){
                	(void) wprintw(stdscr, "Error creating helpwin");
                	(void) refresh();
			LOG_CALL(1);
                	return 0;
        	}

                draw_entrywin(viewwin, abs_path_name, name_value, count);
		
		do{
			key=getch();
			if(key == 'I' || key == 'i')
				break;		
			if(key == ESC)
				break;
		}		
		while(TRUE);

		if(key == 'i' || key == 'I'){
			int buf_count=0;
			/* typing field after the length of field in question to avoid mangling */
			entryfield[0] = new_field(1, (strlen(abs_path_name)+20), 10, (strlen(abs_path_name)+strlen(name_value)+10), 0, 0);
                	entryfield[1] = NULL;
		
			if(entryfield[0] == NULL){
				(void) wprintw(stdscr, "Error creating entry field");
				(void) refresh();
				LOG_CALL(1);
				return 0;
			}
			(void) set_field_back(entryfield[0], A_BOLD);
                	entryform = new_form(entryfield);
			if(entryform == NULL){
				(void) wprintw(stdscr, "Error creating entry form");
				(void) refresh();
				LOG_CALL(1);
				return 0;
			}
			(void) post_form(entryform);
			(void) refresh();
			(void) set_current_field(entryform, entryfield[0]);
			(void) mvprintw(10, 10, "%s: ", abs_path_name);
			draw_entrywin(viewwin, abs_path_name, name_value, count);
			draw_helpwin(helpwin, name);
			draw_lkcpwin(lkcpwin);
			draw_menuwin(menuwin);

			(void) refresh();
			
			while(TRUE){
				(void) curs_set(TRUE);
				key=getch();
				if (key == '>'){ /* commit value to the field, but still check for empty buffers*/
					do{ hold_user_val--; buf_count--;}while(buf_count>0);
					commit_data_field(abs_path_name, hold_user_val);
					(void) werase(viewwin);
                                        (void) werase(modewin);
                                        (void) werase(helpwin);
                                        (void) unpost_form(entryform);
                                        (void) free_form(entryform);
                                        (void) free_field(entryfield[0]);
                                        (void) free_field(entryfield[1]);
                                        sfree(name_value);
                                        sfree(abs_path_name);
					(void) curs_set(FALSE);
					LOG_CALL(1);
					return 1;

				}
				else if(key ==ESC){ /* user changed his mind about comitting data. bail out */
			 		(void) werase(viewwin);
                                	(void) werase(modewin);
                                	(void) werase(helpwin);
					(void) unpost_form(entryform);
					(void) free_form(entryform);
					(void) free_field(entryfield[0]);
					(void) free_field(entryfield[1]);
                                	(void) touchwin(stdscr);
					(void) curs_set(FALSE);
					LOG_CALL(1);
                                	return 1;
				}
				else if(key == '<'){
					(void) form_driver(entryform,REQ_LEFT_CHAR);
					(void) form_driver(entryform, REQ_DEL_CHAR);
					if(buf_count > 0){
						buf_count--;
						hold_user_val--;
						memset(hold_user_val, '\0', 2);
					}
					(void) refresh();
				}
				else {  /* user typed something _relevant_ */
					if(buf_count >= MAX_INPUT_BUFFER){  /* increase this with care for more input */
#ifdef DEBUG
						fprintf(f, "MAX_INPUT_BUFFER reached\n");
#endif
						continue;
					}
					else if(isprint(key)){
						*hold_user_val=key;
						hold_user_val++;
						buf_count++; 
						(void) form_driver(entryform, key);
					}
				}
			}
			
			sfree(name_value);
                 	sfree(abs_path_name);
                        (void) werase(viewwin);
                        (void) werase(modewin);
                        (void) werase(helpwin);
                        (void) touchwin(stdscr);
			(void) curs_set(FALSE);
			LOG_CALL(1);
                        return 1;
		}
		else if(key == ESC){
				sfree(name_value);
				sfree(abs_path_name);
				(void) werase(viewwin);
                 		(void) werase(modewin);
                 		(void) werase(helpwin);
                 		(void) touchwin(stdscr);
				LOG_CALL(1);
                 		return 1;

		}
		(void) touchwin(stdscr);
		LOG_CALL(1);
		return 1;
	}

	sfree(name_value);
	sfree(abs_path_name);
	(void) delwin(viewwin);
	(void) delwin(modewin);
	(void) delwin(helpwin);
	(void) curs_set(FALSE);
	LOG_CALL(1);
	return -1;

}

void draw_entrywin(WINDOW* viewwin, char* name_path, char* name_value, int count){

	char* mode_banner = (char*) malloc(100+1);
	char* value = (char*) malloc(count+1);
	LOG_CALL(0);	
	if(mode_banner == NULL || value == NULL){
		LOG_CALL(1);
		return;	
	}	
	memset(mode_banner, '\0', 100+1);
	memset(value, '\0', count+1);
	
	strncpy(value, name_value, count);
	
	if(!name_path || !viewwin){
                (void) wprintw(stdscr, "draw_viewwin error");
		LOG_CALL(1);
                finish(0);
        }

	(void) box(viewwin, ACS_VLINE, ACS_HLINE);
	(void) wbkgd(viewwin, COLOR_PAIR(12));
        (void) wmove(viewwin, 1, 1);
        (void) wprintw(viewwin, "%s: ", name_path);
	(void) wprintw(viewwin, "%s", value);

	if(lkcp_mode == VIEW_MODE){
		int maxy=0, maxx=0;
		getmaxyx(stdscr, maxy, maxx);
		if(maxy < 3 || maxx < (strlen(mode_banner)+73)){
				nonfatal_error_screen(stdscr);
				LOG_CALL(1);
				return;	
		}	
		strcpy(mode_banner, "View Mode: ESC to exit");	
		modewin = subwin(stdscr, 3, strlen(mode_banner)+73, 12, 0);  /* was 12 */
		(void) box(modewin, ACS_VLINE, ACS_HLINE);
		
	}
	else {
		int maxy=0, maxx=0;
                getmaxyx(stdscr, maxy, maxx);
                if(maxy < 3 || maxx < strlen(mode_banner)+2){
                                nonfatal_error_screen(stdscr);
				LOG_CALL(1);	
                                return;
                }
		strcpy(mode_banner, "Edit Mode: ESC to exit | 'I' or 'i' To change |'>' "
				"to commit change | '<' to delete character");
		modewin = subwin(stdscr, 3, strlen(mode_banner)+2, 12, 0);
		(void) box(modewin, ACS_VLINE, ACS_HLINE);
	}
	(void) wmove(modewin, 1,1);
	(void) wattron(modewin, A_BOLD|COLOR_PAIR(1));
	(void) wprintw(modewin, mode_banner);
	(void) wattroff(modewin, A_BOLD|COLOR_PAIR(1));
	(void) wrefresh(modewin);
	(void) wrefresh(viewwin);	
	sfree(mode_banner);
	sfree(value);

	LOG_CALL(1);	
}

void stripoff(char* value){
	char* temp_value = (char*) malloc(100+1);
	int i = strlen(value);
	LOG_CALL(0);
	if(i == 0){ /* checking the length of the value not the value itself */
		LOG_CALL(1);
		return;
	}
	if(temp_value == NULL){
		LOG_CALL(1);
		return;
	}
	memset(temp_value, '\0', 100+1);

	strncpy(temp_value, value, strlen(value));
	
	do{
		if(isspace(*temp_value))
			*temp_value = ' ';
		temp_value++;
		i--;
	}
	while(i > 0);
	i = 0;
	do {
		temp_value--;
		i++;
	}
	while(i != strlen(value));
	
	strncpy(value, temp_value, strlen(temp_value));
	sfree(temp_value);
}

void commit_data_field(const char* proc_file, const char* value){ /* write value to proc file */
	int chr_count=0, w_count;
	int fd;
	LOG_CALL(0);

	if((fd = open(proc_file, O_RDWR|O_NONBLOCK)) <0){
			LOG_CALL(1);
			return;
	}
	chr_count = strlen(value);
	w_count=write(fd, value, chr_count);
	if (w_count <0){ 	/* add some error handling in case value isn't written */
		LOG_CALL(1);
		return;
	}
	else if(w_count <chr_count){
		(void) wmove(stdscr, 25, 11);
		(void) wprintw(stdscr, "Warning: value trunctacted...");
	}

	log_cmd(proc_file, value);	/* it is safe to assume that the CMD was written to the 
					 * respective kernel data structure. Commit. 
					 */
	(void) close(fd);
	LOG_CALL(1);
}
