/*
-----------------------------------------------------------
 Copyright (c) Youssef Hmamouche - All Rights Reserved.

 This software is subject to the General Public License.
 ----------------------------------------------------------
*/
#include <stdio.h>
#include <curses.h>
#include <signal.h>
#include <menu.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <glib.h>

#include "lkcp.h"
#include "entries.h"
#include "dbg.h"
#include "acknowledge.h"
#include "search.h"
#include "commit_script.h"
#include "scr_err_hndl.h"
#include "sys_name.h"

int lkcp_mode;

char *options = "Copyright (C) Youssef Hmamouche LKCP";
char *banner;

WINDOW *menuwin, *bgrdwin, *lkcpwin;
WINDOW* acknowledge;
WINDOW* viewwin;
WINDOW* modewin;
WINDOW* helpwin;
WINDOW* fieldwin;
WINDOW* debugwin;
WINDOW* searchwin;
WINDOW* dumpwin;
WINDOW* dumpban;

#ifdef DEBUG
extern FILE* f;
#endif

extern GList* cmd_list;
extern char* script_path;
extern int dump_success;

void init();

int main(int argc, char *argv[])
{
	uid_t uid;
	char k; 
	int row=0, col=0;
	int c, ignore_sysname=0;	/* check sysname */
	cmd_list = NULL;

#ifdef DEBUG
	create_logfile(f);
	f = fopen("/tmp/lkcp.log", "a");
        if(!f){
                 printf("Error creating log file: %s\n", strerror(errno));
#undef DEBUG
        }

#endif
	while ((c = getopt (argc, argv, "hvVf")) != -1)
		switch (c){
			case 'v':
			case 'V':
				printf("Live Kernel Configuration Panel %s Copyright "
						"(C) Youssef Hmamouche <hyoussef@gmail.com>\n", LKCP_VERSION);
				exit(0);
				break;
			case 'f':
				ignore_sysname = 1;
				break;
			case 'h':
				print_menu(argv[0]);
				exit(0);
			      	break;	
			case '?':
				print_menu(argv[0]);	
				exit(0);
				break;
			default:
				print_menu(argv[0]);
				exit(0);
	}

	if((uid=getuid()) != 0){
		lkcp_mode = VIEW_MODE;
	}
	else
		lkcp_mode = EDIT_MODE;

	/* cheap way of checking for 'uname -s' == "Linux" */
	
	if (!ignore_sysname){
		if(!is_linux_sys()){
			printf("It appears that this is not a Linux system. "
					"Run LKCP with -f to avoid this check.\n");
			exit(1);
		}
	}

	banner = (char*) malloc(strlen(options)+1);
		
	if(banner == NULL)
		return -1;

	memset(banner, '\0', strlen(options)+1);

	strncpy(banner, options, strlen(options));
    
	init(); /* screen initialization */
	
	getmaxyx(stdscr, row, col);	/* make sure screens don't span beyond the root screen */

	if(col < 100){
		(void) move(10,10);
		(void) wattron(stdscr, COLOR_PAIR(1));
		(void) wprintw(stdscr, "Your terminal's size is too small. Try the command "
			       "\"reset\" at your shell prompt. And try again");
		(void) wattroff(stdscr, COLOR_PAIR(1));
		(void) move(10, 12);
		(void) wprintw(stdscr, "Exiting LKCP...\n");
		(void) refresh();
		finish(1);
	}

	 /* check if current terminal can handle this size or subwin returns NULL*/	
	if(row < 20 || col <100){
		nonfatal_error_screen(stdscr);
        }

	acknowledge = subwin(stdscr, 20, 100, 1, 1);
	
	if(acknowledge == NULL) /* ackowledge is not a necessary screen (nonfatal) */
		nonfatal_error_screen(stdscr);	
	else {
		draw_acknowledge(acknowledge);
		kill_acknowledge(acknowledge);
	}
   
	if(row < 3 || (col < (strlen(banner)+strlen(LKCP_VERSION)+53))){
		fatal_error_screen();
	}
	
	lkcpwin = subwin(stdscr, 3,(int) (strlen(banner)+strlen(LKCP_VERSION)+53),0,0 ); 
	menuwin = subwin(stdscr, 3, (int) (strlen(banner)+strlen(LKCP_VERSION)+53),3,0); 
	/* subwins' size is relative to stdscr. No check */
	bgrdwin = subwin(stdscr, 1, col-6, 20, 2);

	/* each of these subwins are necessary. Bail out if one of them fails to be allocated */
	if(lkcpwin == NULL || menuwin == NULL || bgrdwin == NULL)
		fatal_error_screen();    

	draw_lkcpwin(lkcpwin);
	draw_menuwin(menuwin);

	(void) refresh();

	if(chdir(MAIN_DIR) != 0){
		perror("cannot chdir(MAIN_DIR)");
		finish(1);
	}

	/**
	 * mainwhile is the heart of LKCP. Every time you see the menu prompt, 
	 * you are exactly at k=getch() line.
	 */

	mainwhile: do {

	int selected_item=0;
	(void) curs_set(FALSE);
	(void) wrefresh(bgrdwin);
        (void) werase(stdscr);
        draw_lkcpwin(lkcpwin);
        draw_menuwin(menuwin);
	(void) refresh();
	
	k = getch();

        if (k=='P' || k=='p') {
		DIR* proc=NULL;
		WINDOW** menu_items;
		menu_items=draw_menu(4);
		if(menu_items == NULL){
			nonfatal_error_screen(stdscr);
			goto mainwhile;
		}
		
		selected_item=scroll_menu(menu_items,9,4);
		if(selected_item <0){	
			delete_menu(menu_items,	15);
			(void) touchwin(stdscr);
			(void) refresh();
			goto mainwhile;
	    	}
            	delete_menu(menu_items,10);

		/* This is the second most important while loop in lkcp_main.c */
	    	
		while((proc = opendir(dent[selected_item+1].names)) != NULL){
			int ret;

        		if((ret=chdir(dent[selected_item+1].names)) <0){
                		(void) wprintw(stdscr, "Error chdir() -> 2nd while");
                		(void) refresh();
				closedir(proc);
                		finish(0);
        		}
            	
			if (selected_item<0){
				continue;
			}
	    		else {	/* move through the /proc directory */
	
				WINDOW** proc_sys_items;

				proc_sys_items=draw_sys_menuwin(proc, 30);

				if(proc_sys_items == NULL){
					nonfatal_error_screen(stdscr);
					closedir(proc);
					goto mainwhile;
				}	
				selected_item=scroll_menu(proc_sys_items, dent[0].entry, 6);
				if(selected_item <0){
                                	delete_menu(proc_sys_items, (dent[0].entry)+1);
                                	(void)touchwin(stdscr);
                                	(void) refresh();
					closedir(proc);
                                	goto mainwhile;
                        	}
		
				if(selected_item>=0){
					if(strncmp(dent[1].names, "<empty>", 4) == 0){
						(void) touchwin(stdscr);
						delete_menu(proc_sys_items, (dent[0].entry)+1);
                                		(void) refresh();
						closedir(proc);
                                		goto mainwhile;
					}
	
				}
	    		}
			closedir(proc);
      			(void) touchwin(stdscr);
           		(void) refresh();
		} /* while */
		
		if (errno == ENOTDIR){
			int ret;

			if((ret=editentry(dent[selected_item+1].names)) > 0){ /* success from a permission point of view*/
				/* root mode: value saved */
				 draw_lkcpwin(lkcpwin);
                                 draw_menuwin(menuwin);
			}
			else if(ret == 0){ /* recoverable execution */
				goto mainwhile;
			}
			else if(ret == -1){ /* fatal error */
				/* user mode: value not saved due to permission or mem error*/	
				(void) wprintw(stdscr, "Fatal memory error detected.");
				(void) refresh();
				finish(0);	
			}
		}
		else if(errno == ENOENT){
			(void) wmove(stdscr, row-20, 1);
			(void) wprintw(stdscr, "Directory %s does not exist.", dent[selected_item+1].names);
			(void) refresh();
			(void) sleep(1);
			/* ugly but costs less than an erase */
			(void) wmove(stdscr, row-20, 1);
			(void) wprintw(stdscr, "                                                         ");
			(void) touchwin(stdscr);
			(void) refresh();
			goto mainwhile;
		}
		else {
			(void) wprintw(stdscr, "Error openning file %s", dent[selected_item+1].names);
			(void) refresh();
			goto mainwhile;
		}
        } 
	else if(k=='E' || k=='e'){	/* exit LKCP */
		break;
	}
	else if(k=='S' || k=='s'){	/* search option */
		int maxcols=0, maxrows=0;
		char* pattern = (char*) malloc(MAX_INPUT_BUF+1);
		if (pattern == NULL)
			goto mainwhile;
		memset(pattern, '\0', MAX_INPUT_BUF+1);
		getmaxyx(stdscr,maxrows, maxcols);
		if(maxrows < maxrows-10-10 ){
			nonfatal_error_screen(stdscr);
		}	
		searchwin = subwin(stdscr, maxrows-10, maxcols-1, 10, 0);
		if(searchwin == NULL){
			nonfatal_error_screen(stdscr);
			sfree(pattern);
			goto mainwhile;
		}
		draw_searchwin(searchwin);
		pattern=input_pattern();
		if(pattern == NULL)
			goto mainwhile;
		search_pattern(pattern);
		draw_search_menuwin(pattern);
		sfree(pattern);
		goto mainwhile;
	}
	else if(k == 'D' || k == 'd'){	/* dump a scriptable script proc_commit-`date %H-%M-%S`*/
		int maxcols=0, maxrows=0;
		dump_success=0;
		getmaxyx(stdscr, maxrows, maxcols);
		if(maxcols <(strlen(banner)+strlen(LKCP_VERSION)+53)){	/* ugly hack to have dumpwin in sync with lkcpwin */
			nonfatal_error_screen(stdscr);
			goto mainwhile;
		}
		dump_script();
		dumpwin = subwin(stdscr, 3, (strlen(banner)+strlen(LKCP_VERSION)+53), 15, 0);
		if(dumpwin == NULL){
			nonfatal_error_screen(stdscr);
			goto mainwhile;
		}
		dumpban = subwin(stdscr, 3, (strlen(banner)+strlen(LKCP_VERSION)+53), 12, 0);
		if(dumpban == NULL){
			nonfatal_error_screen(stdscr);
			goto mainwhile;
		}
		draw_dumpban(dumpban);
		draw_dumpwin(dumpwin);
		while(getch() != ESC);
		goto mainwhile;
	}
    } while(1);	/* mainwhile end */

finish(0);

return 0;
}

/**
 * finish() is a callback function and is hooked up to signals in init(). This 
 * function does the necessary cleanups such as screen window, temporary files, 
 * environment variables, etc... 
 */
 
void finish(int sig){

	(void) wattron(bgrdwin, COLOR_PAIR(1));
	(void) wprintw(bgrdwin, "Exiting LKCP (SIGNAL %d)...", sig);
	(void) touchwin(stdscr);
	(void) refresh();
	(void) delwin(bgrdwin);
	(void) delwin(menuwin);
	(void) delwin(lkcpwin);
	(void) delwin(acknowledge);
	sfree(banner);	
	(void) unlink((char*) getenv("LKCP"));
	(void) unsetenv("LKCP");

	sleep(1);
	if( endwin() != 1)
		(void) wprintw(stdscr, "exiting abruptly...");

	exit(0);
}

void init(){

	(void) signal(SIGINT, finish); 
	(void) signal(SIGSEGV, finish);	
	(void) signal(SIGILL, finish);
	(void) signal(SIGABRT, finish);
	(void) signal(SIGINT, finish);
	(void) signal(SIGTRAP, SIG_IGN);

	(void) initscr();      /* initialize the curses library */

	(void) curs_set(FALSE);
	keypad(stdscr, TRUE);  /* enable keyboard mapping */
	(void) nonl();         /* tell curses not to do NL->CR/NL on output */
	(void) cbreak();       /* take input chars one at a time */
	(void) noecho();       /* don't echo input */
	(void) meta(stdscr, TRUE);
	(void) meta(lkcpwin, TRUE);
	(void) meta(menuwin, TRUE);
	(void) meta(bgrdwin, TRUE);
	(void) meta(viewwin, TRUE);
	(void) meta(modewin, TRUE);
	(void) meta(helpwin, TRUE);
	(void) meta(fieldwin, TRUE);
	(void) meta(acknowledge, TRUE);
	(void) meta(searchwin, TRUE);
	(void) meta(dumpwin, TRUE);
	(void) meta(dumpban, TRUE);

	if (has_colors()){

		(void) start_color();
		
		(void) init_pair(1, COLOR_RED, COLOR_BLACK);
		(void) init_pair(7, COLOR_WHITE, COLOR_BLUE);
		(void) init_pair(8, COLOR_WHITE, COLOR_BLACK);
		(void) init_pair(10, COLOR_BLACK, COLOR_WHITE);
		(void) init_pair(12, COLOR_BLACK, COLOR_CYAN);
    }

	(void) curs_set(FALSE);
	/* init debugging window*/

	if(!stdscr){
#ifdef DEBUG
		LOG_CALL(1);
		fprintf(f, "stdscr is NULL. Something is fishy!");
#endif
		finish(EXIT_FAILURE);
	}

}

void draw_lkcpwin(WINDOW* lkcpwin){

	int maxy=0, maxx=0;
	getmaxyx(lkcpwin, maxy, maxx);
	(void) box(lkcpwin, ACS_VLINE, ACS_HLINE);
	(void) wbkgd(lkcpwin, COLOR_PAIR(1));
	(void) wmove(lkcpwin, 1, (maxx/3)-5);
	(void) wattron(lkcpwin, A_BOLD);
	(void) wprintw(lkcpwin, "%s %s", banner, LKCP_VERSION);
	(void) wattroff(lkcpwin, A_BOLD);
	(void) wrefresh(lkcpwin);
}

void draw_menuwin(WINDOW* menuwin){

	(void) wbkgd(menuwin,COLOR_PAIR(1));
	(void) box(menuwin, ACS_VLINE, ACS_HLINE);
	(void) wmove(menuwin, 1,1);
	(void) wattron(menuwin, A_BOLD);
	(void) wattron(menuwin,COLOR_PAIR(8));
	(void) waddstr(menuwin, "P");
	(void) wattroff(menuwin,COLOR_PAIR(8));
	(void) waddstr(menuwin, "roc config");
	(void) wmove(menuwin,1,25);
	(void) wattron(menuwin, A_BOLD);
	(void) wattron(menuwin, COLOR_PAIR(8));
	(void) waddstr(menuwin, "S");
	(void) wattroff(menuwin, COLOR_PAIR(8));
	(void) waddstr(menuwin, "earch pattern");
	(void) wmove(menuwin,1,50);
	(void) wattron(menuwin, A_BOLD);
        (void) wattron(menuwin, COLOR_PAIR(8));
	(void) waddstr(menuwin, "D");
	(void) wattroff(menuwin, COLOR_PAIR(8));
	(void) waddstr(menuwin, "ump script");
        (void) wmove(menuwin,1,80);
	(void) wattron(menuwin, COLOR_PAIR(8));
	(void) waddstr(menuwin, "E");
	(void) wattroff(menuwin,COLOR_PAIR(8));
        (void) waddstr(menuwin, "xit");
	(void) wattroff(menuwin, A_BOLD);
	(void) wrefresh(menuwin);
}

WINDOW **draw_menu(int start_col){
        
	int i, maxy=0, maxx=0;
        WINDOW **items;
        items=(WINDOW **)malloc(10*sizeof(WINDOW *));
	getmaxyx(stdscr, maxy, maxx);
	if(maxy < 11 || maxx <25){
		fatal_error_screen();
	}
        items[0]=newwin(11,25,6,start_col);
        (void) wbkgd(items[0],COLOR_PAIR(10));	/* was 8 */
        (void) box(items[0],ACS_VLINE,ACS_HLINE);

	for(i = 1; i<10; i++){ 
        	items[i]=subwin(items[0],1,20,6+i,start_col+1);
	}
	for(i = 1; i<dent[0].entry; i++){
		memset((dent[i].names), '\0', sizeof(dent[i].names));
	}
       	strncpy((dent[1]).names, PROC_ABI, sizeof(dent[1].names));
	strncpy((dent[2]).names, PROC_FS, sizeof(dent[2].names));
	strncpy((dent[3]).names, PROC_NET, sizeof(dent[3].names));
	strncpy((dent[4]).names, PROC_DEBUG, sizeof(dent[4].names));
	strncpy((dent[5]).names, PROC_VM, sizeof(dent[5].names));
	strncpy((dent[6]).names, PROC_KERNEL, sizeof(dent[6].names));
	strncpy((dent[7]).names, PROC_SUNRPC, sizeof(dent[7].names));
	strncpy((dent[8]).names, PROC_DEV, sizeof(dent[8].names));
	strncpy((dent[9]).names, PROC_PROC, sizeof(dent[9].names));
	
	for(i = 1; i <10; i++){	
		(void) wprintw(items[i], "%s ->", dent[i].names);
	}
        (void) wbkgd(items[1],COLOR_PAIR(12));	/* was 11 */
        (void) wrefresh(items[0]);
        return items;
}

int scroll_menu(WINDOW **items,int count,int menu_start_col){

        int key, selected=0;
        while (1) {
		key=getch();
                if (key==KEY_DOWN || key==KEY_UP) {
                        (void) wbkgd(items[selected+1],COLOR_PAIR(10));
                        (void) wnoutrefresh(items[selected+1]);
                        if (key==KEY_DOWN) {
                                selected=(selected+1) % count;
                        } 
			else {
                                selected=(selected+count-1) % count;
                        }
			
                        (void) wbkgd(items[selected+1],COLOR_PAIR(12));
                        (void) wnoutrefresh(items[selected+1]);
                        (void) doupdate();
                } 
		else if(key==KEY_RIGHT){
				(void) doupdate();
                                return selected;
                        }
		else if(key==KEY_LEFT){
				(void) doupdate();
				return -1;
		}
					
		else if (key==ESC) {
                        return -1;
                } 
		
             }
}

void delete_menu(WINDOW **items,int count){

        int i; for (i=0;i<count;i++)
                (void) delwin(items[i]);
        sfree(items);
}

void print_menu(char* progname){
	
	printf("Usage: %s [OPTION]...\n", progname);
	printf("\t-h\t\tprint this help menu\n");
	printf("\t-v\t\tprogram version and info\n");
	printf("\t-f\t\tignore system name checking\n");
}
