/*
-----------------------------------------------------------
 Copyright (c) Youssef Hmamouche - All Rights Reserved.

 This software is subject to the General Public License.
 ----------------------------------------------------------
*/

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <errno.h>
#include "lkcp.h"
#include "dbg.h"
#include "scr_err_hndl.h"

extern WINDOW* bgrdwin;

extern void finish(int i);

#ifdef DEBUG
extern FILE* f;
#endif

extern directent dent[MAX_RECURS_DIR];

WINDOW** draw_sys_menuwin(DIR* dir, int start_col){
        WINDOW **items;
	struct dirent* entry;
	struct stat stats;
	unsigned int i=1, maxy, maxx;
	char* dir_abi = (char*) malloc(100+1);
	char* current_dir = (char*) get_current_dir_name();
	LOG_CALL(0);	
	memset(dir_abi, '\0', 100+1);
	
	if(dir == NULL){
		(void) wprintw(stdscr, "draw_sys error dir");
		LOG_CALL(1);
		finish(0);
	}
	if(dir_abi == NULL){
		LOG_CALL(1);
		return NULL;
	}
	
	if(current_dir == NULL){
		LOG_CALL(1);
		return NULL;
	}

	strncpy(dir_abi, current_dir, strlen(current_dir));
	set_number_entries(dir_abi);	

	getmaxyx(stdscr, maxy, maxx);
	if((dent[0].entry)+2 >= maxy){
		dent[0].entry = maxy-6-3; /* 6 because start column for  items[0] is 6 and 3 for not stepping on edges */	
	}
	if((dent[0].entry) <0){
		LOG_CALL(1);
		return NULL;
	}

	items=(WINDOW **)malloc(((dent[0].entry)+2)*sizeof(WINDOW *));

	if(items==NULL){
                (void) wprintw(stdscr, "draw_sys_menuwin(): error allocating items");
                (void) refresh();
		LOG_CALL(1);
                finish(0);
        }
	
	if(maxy < (dent[0].entry)+2){ 	/* if you modify elements in newwin below make sure to*/
		LOG_CALL(1);				/* set those values here accordingly */
		fatal_error_screen();			/* function calls finish*/
	}

	items[0]=newwin((dent[0].entry)+2,28, 6, start_col); /* what if dent[0].entry > max_row? lkcp crashes */
							     /*dent[0].entry gets updated by set_number_entries */

	if(items[0] == NULL){	/* update: it shouldn't crash anymore. lkcp should exit gracefully */
		if(stdscr == NULL){
			LOG_CALL(1);
			finish(0);
		}
		nonfatal_error_screen(stdscr);
		sfree(dir_abi);
		LOG_CALL(1);
		return NULL;
	}
	
        wbkgd(items[0],COLOR_PAIR(10));	/* was 8 */
        box(items[0],ACS_VLINE,ACS_HLINE);
	
	if(dir == NULL){
		(void) wprintw(stdscr, "Error openning %s", dir_abi);
		(void) refresh();
		LOG_CALL(1);
		finish(0);
	}

	while((entry = readdir(dir)) !=NULL){
		int statret;
		if(i > dent[0].entry){
			(void) mvwprintw(stdscr, maxy, 1, "This is a partial listing.");
			(void) refresh();
			break;	
		}
		if( (statret=lstat(entry->d_name, &stats)) < 0){
			(void) wprintw(stdscr, "lstat: %d", statret);
			LOG_CALL(1);
			finish(1);
		}
		if(strcmp(entry->d_name, ".") == 0 || 
		   strcmp(entry->d_name, "..") == 0)
				continue;	

		memset(dent[i].names, 0, 100);
		/*strncpy((dent[i]).names, entry->d_name, _D_EXACT_NAMLEN(entry));*/
		strncpy((dent[i]).names, entry->d_name, strlen(entry->d_name));
		if(S_ISDIR(stats.st_mode)){
				items[i]=subwin(items[0],1, 25,6+i,start_col+1);
	                	(void) wprintw(items[i],"%s ->", (dent[i]).names);
			
			}
		else if (strlen(entry->d_name) > 0){

			getmaxyx(items[0], maxy, maxx);
			if(maxy < 1 || maxx < 28){
				LOG_CALL(1);
                		return NULL; /* goto mainwhile*/
        		}
			items[i]=subwin(items[0],1,25,6+i,start_col+1);
       			(void) wprintw(items[i],"%s", (dent[i]).names);
		}
		else
			continue;	

		i++;		
	}
	
	if(i-1 == 0){

		strncpy(dent[1].names, "<empty>", strlen((char*) "<empty>"));
		items[1]=subwin(items[0], 2, 25, 7, start_col+1);
		(void) wprintw(items[1], "%s", dent[1].names);
		(void) refresh();
		dent[0].entry++;
        }

     	wbkgd(items[1],COLOR_PAIR(12));	/* was 1 */
        (void) wrefresh(items[0]);
	
	sfree(dir_abi);
	LOG_CALL(1);
        return items;
	
}

void set_number_entries(char* dir){
	DIR* abi;
        int i=0;
        struct dirent* entry;
        struct stat stats;
        char* dir_abi = (char*) malloc(100 * sizeof(dir_abi));
	LOG_CALL(0);
	dent[0].entry_len=0;

	if(dir_abi == NULL){
                (void) wprintw(stdscr, "WARNING: dir_abi==NULL");
                (void) refresh();
		dent[0].entry= -1;
		LOG_CALL(1);
		return;
        }

	strcpy(dir_abi, dir);

	abi = opendir(dir_abi);

        if(abi == NULL){
                (void) wprintw(stdscr, "Error openning %s", dir_abi);
                (void) refresh();
		dent[0].entry=-1;
		LOG_CALL(1);
                return;
        }

	if( chdir(dir_abi) < 0){
		(void) wprintw(stdscr, "Can't change to dir %s", dir);
		(void) refresh();
		dent[0].entry=-1;
		LOG_CALL(1);
		return;
	}

        while((entry = readdir(abi)) !=NULL){

                if(lstat(entry->d_name, &stats) <0){
			dent[0].entry = -1;
			LOG_CALL(1);
			return;
		}
                if(strcmp(entry->d_name, ".") == 0 ||
                   strcmp(entry->d_name, "..") == 0)
                                continue;
		if(dent[0].entry_len < strlen(entry->d_name))
                        dent[0].entry_len = strlen(entry->d_name);

                i++;
        }

        dent[0].entry = i;

	rewinddir(abi);
	sfree(dir_abi);
	sfree(entry);
	sfree(abi);
	LOG_CALL(1);
}
