/*
-----------------------------------------------------------
 Copyright (c) Youssef Hmamouche - All Rights Reserved.

 This software is subject to the General Public License.
 ----------------------------------------------------------
*/

#include "lkcp.h"
#include "dbg.h"
#include "scr_err_hndl.h"

extern WINDOW* lkcpwin;
extern WINDOW* menuwin;
extern WINDOW* bgrdwin;
extern WINDOW* viewwin;
extern WINDOW* modewin;
extern WINDOW* helpwin;
extern WINDOW* fieldwin;
extern WINDOW* debugwin;
extern WINDOW* acknowledge;

#ifdef DEBUG
extern FILE* f;
#endif

void fatal_error_screen(void){
	WINDOW* errorwin;
	int maxy=0,maxx=0;
	LOG_CALL(0);
	if(lkcpwin != NULL)
		(void) werase(lkcpwin);
	if(menuwin != NULL)
		(void) werase(menuwin);
	if(bgrdwin != NULL)
		(void) werase(bgrdwin);
	if(viewwin != NULL)
		(void) werase(viewwin);
	if(modewin != NULL)
		(void) werase(modewin);
	if(helpwin != NULL)
		(void) werase(helpwin);
	if(fieldwin != NULL)
		(void) werase(fieldwin);
	if(debugwin != NULL)
		(void) werase(debugwin);
	if(acknowledge != NULL)
		(void) werase(acknowledge);
	(void) getmaxyx(stdscr, maxy, maxx);
	if(maxy >= 5 || maxx >= 30)
		errorwin = subwin(stdscr, 5, 40, 0, 0);
	else
		errorwin = subwin(stdscr, 2, 10, 0, 0); 	/* Let's hope it will return a window */
	if(errorwin == NULL){					/* There must be a proper way of alterting the user */
		(void) wprintw(stdscr, "Error creating screen errorwin");
		(void) sleep(3);
		LOG_CALL(1);
		finish(1);
	}
	meta(errorwin, TRUE);
	
	(void) wattron(stdscr, A_BOLD);
        (void) wattron(stdscr, A_UNDERLINE);

        (void) wmove(stdscr, 1, 1);
        (void) wprintw(stdscr, "LKCP cannot run. Adjust terminal size");      
        (void) refresh();
        (void) sleep(5);
        (void) wclear(stdscr);
        (void) wrefresh(stdscr);

        (void) wattroff(stdscr, A_BOLD);
        (void) wattroff(stdscr, A_UNDERLINE);
	LOG_CALL(1);	
	finish(1);

}

/*
	If the current screen is erroring out, get the parent to output this error. 
	It costs less to compute this than to have ncurses create a new window 
*/
void nonfatal_error_screen(WINDOW* current){
	int maxy=0, maxx=0;
	LOG_CALL(0);
	if(current == NULL){
		fatal_error_screen();
		LOG_CALL(1);
		return;
	}

		(void) getmaxyx(current, maxy, maxx);
		(void) werase(current);

		(void) wattron(current, A_BOLD);
                (void) wattron(current, A_UNDERLINE);
                (void) wattron(current, COLOR_PAIR(1));

                (void) wmove(current, maxy/2, maxx/8);
                (void) wprintw(current, "Error displaying screen. Resetting LKCP...");
                (void) wrefresh(current);
                (void) sleep(3);
                (void) wclear(current);
                (void) wrefresh(current);
                (void) wattroff(current, COLOR_PAIR(1));
                (void) wattroff(current, A_BOLD);
                (void) wattroff(current, A_UNDERLINE);

                draw_lkcpwin(lkcpwin);
                draw_menuwin(menuwin);
                (void) refresh();
		LOG_CALL(1);
}
		

void draw_errorwin(WINDOW* errorwin){
	LOG_CALL(0);
        (void) box(errorwin, ACS_VLINE, ACS_HLINE);
        (void) wbkgd(errorwin, COLOR_PAIR(1));
        (void) wrefresh(errorwin);
	LOG_CALL(1);
}

