/*
-----------------------------------------------------------
 Copyright (c) Youssef Hmamouche - All Rights Reserved.

 This software is subject to the General Public License.
 ----------------------------------------------------------
*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <libgen.h>
#include <fcntl.h>
#include <errno.h>
#include "search.h"
#include "lkcp.h"
#include "dbg.h"
#include "entries.h"
#include "scr_err_hndl.h"

extern WINDOW* lkcpwin;
extern WINDOW* menuwin;
extern WINDOW* searchwin;

FORM* srchform;
FIELD* srchfield[2];

const char* input_field_banner = "Enter string to search for: ";

#ifdef DEBUG
extern FILE* f;
#endif

glob_t globbuf;
void draw_searchwin(WINDOW* searchwin){
	LOG_CALL(0);
        (void) wbkgd(searchwin, COLOR_PAIR(8));
        (void) wmove(searchwin, 1, 1);
        (void) wattron(searchwin, A_BOLD);
        (void) wattroff(searchwin, A_BOLD);
        (void) wrefresh(searchwin);
	LOG_CALL(1);
}

char* input_pattern(){
	int key, buf_count=0;
	char* buf=(char*) malloc(4+1);
	char* input = (char*) malloc(100+1);
	const char* search_banner = "Search pattern (at least 3 characters. CTRL-X:Search CTRL-B:Backspace): ";
	
	LOG_CALL(0);
       
	if(input == NULL){
                (void) wprintw(stdscr, "input_pattern(): Memory allocation error");
                (void) refresh();
		 LOG_CALL(1);
                return NULL;
        }

        if(buf == NULL){
                (void) wprintw(stdscr, "input_pattern(): Memory allocation error");
                (void) refresh();
		 LOG_CALL(1);
                return NULL;
        }
 
	memset(input, 0, 100+1);
	memset(buf, 0, 4+1);
	/* second was MAX_INPUT_BUF */
	srchfield[0] = new_field(1, MAX_INPUT_BUF+1, 11, (strlen(search_banner)+1), 0, 0); /*was 50*/
        srchfield[1] = NULL;

        if(srchfield[0] == NULL){
        	(void) wprintw(stdscr, "Error creating search field");
                (void) refresh();
		LOG_CALL(1);
                 return NULL;
        }
        (void) set_field_back(srchfield[0], A_BOLD);
        srchform = new_form(srchfield);
        if(srchform == NULL){
        	(void) wprintw(stdscr, "Error creating search form");
                (void) refresh();
		sfree(buf);
		LOG_CALL(1);
                return NULL;
      	}
	(void) post_form(srchform);
	(void) refresh();
	(void) set_current_field(srchform, srchfield[0]);
	(void) curs_set(TRUE);
	(void) wattron(searchwin, A_BOLD);
	(void) wprintw(searchwin, "%s", search_banner);
	(void) wattroff(searchwin, A_BOLD);
	(void) wrefresh(searchwin);
	draw_lkcpwin(lkcpwin);
        draw_menuwin(menuwin);
	(void) refresh();

	do {
		key=getch();
		*buf = key;
		if(strcmp(buf, "\030")==0){ /* ^X CTRL-X */
			if(buf_count<= 0){
				(void) curs_set(FALSE);
				LOG_CALL(1);
                                return NULL;
			}
			if(buf_count <3){
				(void) curs_set(FALSE);
				LOG_CALL(1);
				return NULL;
			}
			do{ input--; buf_count--;}while(buf_count>0);
			sfree(buf);
                        (void) unpost_form(srchform);
                        (void) free_form(srchform);
                        (void) free_field(srchfield[0]);
                        (void) free_field(srchfield[1]);
			(void) refresh();
			draw_lkcpwin(lkcpwin);
                        draw_menuwin(menuwin);
			(void) curs_set(FALSE);
#ifdef DEBUG
                        fprintf(f, "input: %s\n", input);
#endif

			LOG_CALL(1);
			return input;
		}
		else if(strcmp(buf, "\002")==0){ /* ^B CTRL-B */

				(void) form_driver(srchform,REQ_LEFT_CHAR);
				(void) form_driver(srchform, REQ_DEL_CHAR);
				if(buf_count > 0){
					buf_count--;
					input--;
					memset(input, '\0', 2);
				}
                                (void) refresh();
		}
		else if(key == ESC){
				(void) unpost_form(srchform);
				(void) free_form(srchform);
				(void) free_field(srchfield[0]);
                                (void) free_field(srchfield[1]);
				draw_lkcpwin(lkcpwin);
				draw_menuwin(menuwin);
				(void) curs_set(FALSE);
				LOG_CALL(1);
				return NULL;
		}
		else {
				if(buf_count >= MAX_INPUT_BUF){
					continue;
				}
                                else if(isprint(key)){
                                	*input=key;
                                	input++;
                                	buf_count++;
                                	(void) form_driver(srchform, key);
                              	}
		}	
	}while(1);

	LOG_CALL(1);
}

void search_pattern(const char* pattern){
	pid_t pid;	
	char* str;
	int error;
	char* val;
	char* home = (char*) getenv("HOME");
	LOG_CALL(0);
	if(pattern == NULL){
		LOG_CALL(1);
		return;
	}
			
	if(home == NULL){
		LOG_CALL(1);	
		return;
	}
	
	val = (char*) malloc(strlen(home)+6+1); /* 6 ->"/.lkcp" */
	
	if(val == NULL){
		LOG_CALL(1);
		return;
	}
	
	memset(val, '\0', strlen(home)+6+1);
	strncpy(val, home, strlen(home));
	strncat(val, "/.lkcp", strlen("/.lkcp"));
	if((error=setenv("LKCP", val, 1)) <0){
		printw("Error creating environment variable %s", strerror(error));
		sfree(val);
		LOG_CALL(1);
		finish(1);
	}

	if(open(val, O_RDONLY) != -1){
		if(unlink((char*) getenv("LKCP"))<0){; /*what would happen anyway?*/
			LOG_CALL(1);
		}
	}	/* file doesn't exist good */

	str= (char*) malloc(strlen(pattern)+2+1); /* 2 for opening and closing quotes */
	
	if(str == NULL){
		LOG_CALL(1);
		return;
	}
	memset(str, '\0', strlen(pattern)+2+1);
        
	*str = (char) '\047';
        strncat(str, pattern, strlen(pattern));
        strncat(str, (char*) "\047\0",2); /* end quote and null-terminate string */

	globbuf.gl_offs=6;
	if(glob("", GLOB_DOOFFS, NULL, &globbuf) !=0){
		sfree(val);
		sfree(str);
		LOG_CALL(1);
		return;
	}
	globbuf.gl_pathv[0] = "find";
	globbuf.gl_pathv[1] = "/proc/sys/";
	globbuf.gl_pathv[2] = "-fprint";
  	globbuf.gl_pathv[3] = (char*) getenv("LKCP");
	globbuf.gl_pathv[4] = NULL;

	if( (pid=fork()) <0){
#ifdef DEBUG
		fprintf(f, "error creating process: %s", strerror(errno));
#endif
		LOG_CALL(1);
		return;
	}
	else if(pid == 0){
		if(execvp("find", &globbuf.gl_pathv[0]) <0){
#ifdef DEBUG
			fprintf(f, "error launching find: %s", strerror(errno));
#endif
			LOG_CALL(1);
			return;
		}
	}
	if(waitpid(pid, NULL,0) <0){
		LOG_CALL(1);
#ifdef DEBUG
                fprintf(f, "error while waiting for child process: %s", strerror(errno));
#endif
		return;
	}

	sfree(val);
	sfree(str);
	
	LOG_CALL(1);
}

void draw_search_menuwin(const char* pat){
	WINDOW** search_items=NULL;
	int selected_srch_item=0;
	int ret;
	char* result_file = getenv("LKCP");
	char* abs_path;
	int len;
	LOG_CALL(0);
	
	if(result_file == NULL){
		LOG_CALL(1);
		return;
	}
	get_number_entries(result_file, pat);

	if(srch_item[0].nr_entries <0){
		/*nonfatal_error_screen(stdscr);
		(void) refresh(); */
		LOG_CALL(1);
                return;
        }
	if(srch_item[0].nr_entries == 0){ /* draw searchhelpwin */
		LOG_CALL(1);
		return;
	}
	search_items = draw_search_results(pat);
	
	if(search_items == NULL){
		nonfatal_error_screen(stdscr);
		LOG_CALL(1);
		return;
	}	
	selected_srch_item = scroll_menu(search_items, srch_item[0].nr_entries, 10);

	if(selected_srch_item < 0){
		delete_menu(search_items, (srch_item[0].nr_entries)+1);
		(void) touchwin(stdscr);
                (void) refresh();
		LOG_CALL(1);
		return;
	}	
	len = strlen(srch_item[selected_srch_item+1].fname);

	abs_path = (char*) malloc(len+1);
	memset(abs_path, '\0', len+1);

	strncpy(abs_path, srch_item[selected_srch_item+1].fname, len);
	if(chdir((char*) dirname(abs_path)) !=0){
		(void) wprintw(searchwin, "error changing directory %s", strerror(errno));	
		(void) refresh();
	}

	if(search_items != NULL)
		delete_menu(search_items, (srch_item[0].nr_entries)+1);
	(void) touchwin(stdscr);
	(void) refresh();

	if((ret=editentry((char*) basename(srch_item[selected_srch_item+1].fname))) > 0){ /* success from a permission point of view*/

		/*draw_lkcpwin(lkcpwin);
		draw_menuwin(menuwin); */
        }
	else if(ret == 0){ /* recoverable execution */
		LOG_CALL(1);
		return;
        }
	else if(ret == -1){ /* fatal error */
                               /* user mode: value not saved due to permission or mem error*/
		(void) wprintw(stdscr, "Fatal memory error detected.");
		(void) refresh();
		LOG_CALL(1);
		finish(0);
	}

}
WINDOW** draw_search_results(const char* pattern){
	WINDOW** items=NULL;
	FILE* fp;
	char* lkcp_file = (char*) getenv("LKCP");
	char* line=NULL;
	int biggest_strlen=0;	/* holds the longest field string so we can resize the item win */
	unsigned int i=1, maxx=0, maxy=0, start_col=10;
	ssize_t len = 0;	
	ssize_t read;
	
	LOG_CALL(0);
	
	fp=fopen(lkcp_file, "r");
	if(fp == NULL){
		wmove(stdscr, 10, 10);
		(void) wprintw(stdscr,"fp is null: %s", strerror(errno));
		(void) refresh();
		(void) sleep(3);
	}
		
	getmaxyx(searchwin, maxy, maxx);
	
	if(((srch_item[0].nr_entries)+2) >=maxy || srch_item[0].entry_len>=maxx){
		nonfatal_error_screen(stdscr);
		LOG_CALL(1);
		return NULL;
	}
	items=(WINDOW **)malloc(((srch_item[0].nr_entries)+2)*sizeof(WINDOW *)); /* display only what searchwin can take */

        if(items==NULL){
                (void) wprintw(stdscr, "draw_search_results(): error allocating items");
                (void) refresh();
		LOG_CALL(1);
                finish(0);
        }
        memset(items, '\0',((srch_item[0].nr_entries)+2)*sizeof(WINDOW*));

	if(maxy < (srch_item[0].nr_entries)+2 || maxx <srch_item[0].entry_len){       
		LOG_CALL(1);
		fatal_error_screen();                   /* function calls finish*/
        }
	
	/* srch_item[0].entry_len+2 <= let's not step on the edge of the box */	
        items[0]=newwin(((srch_item[0].nr_entries)+2),(srch_item[0].entry_len)+2, 6, start_col);

        if(items[0] == NULL){
                if(stdscr == NULL){
			LOG_CALL(1);
                        finish(0);
		}
                nonfatal_error_screen(stdscr);
		LOG_CALL(1);
                return NULL;
        }
	(void) refresh();

        wbkgd(items[0],COLOR_PAIR(10));
        /*(void) box(items[0],ACS_VLINE,ACS_HLINE); */
	/*(void) wrefresh(items[0]);*/

	getl: 	while((read=getline(&line, &len, fp)) != -1){
			DIR* d;
			char* l = (char*) malloc(len+1);
 
                        if(l == NULL)
                                goto getl;

			if(len <=0)
				goto getl;
			memset(l, '\0', strlen(line)+1);
			if(read <=0)
				goto getl;
			strncpy(l, line, strlen(line)-1);
			d=opendir(l);
			if(d != NULL){
				closedir(d);	
				sfree(l);	
				goto getl;
			}
			if(d)
				closedir(d);
			if(strstr((l+9), pattern) != NULL){ /* skip /proc/sys/ */
				if(strncpy(srch_item[i].fname, l, strlen(l)) == NULL){
					sfree(l);
					goto getl;
				}
				if(len >0){
					if(i >= maxy-2){
						i--;
						sfree(l);
						goto out;	
					}
					items[i]=subwin(items[0],1, strlen(srch_item[i].fname),6+i,start_col+1); /* was 25 */
					(void) wprintw(items[i],"%s", (srch_item[i]).fname);
					(void) wrefresh(items[i]);
					if(biggest_strlen < strlen((srch_item[i]).fname))
						biggest_strlen=strlen((srch_item[i]).fname);
					i++;
				}
			}
                	sfree(l);

		}
out:
	wbkgd(items[1],COLOR_PAIR(12));
	(void) box(items[0],ACS_VLINE,ACS_HLINE);
	(void) wrefresh(items[0]);
	(void) refresh();

	fclose(fp);	
	LOG_CALL(1);
	return items;
}
void get_number_entries(const char* file, const char* pattern){
	FILE* fp;
	int biggest_strlen=0;
	int i=0, maxy=0, maxx=0;
	char* line;
	ssize_t len = 0;
	ssize_t read;
	LOG_CALL(0);
	fp = fopen(file, "r");
	if(fp == NULL){
		(void) printw("fp null: %s", strerror(errno));
		(void) refresh();
		(void) sleep(2);
		srch_item[0].nr_entries=-1;
		LOG_CALL(1);
		return;
	}
	if(pattern == NULL){
		(void) printw("pattern null: %s", strerror(errno));
		(void) refresh();
		(void) sleep(2);
		srch_item[0].nr_entries=-1;
		LOG_CALL(1);
		return;
	}
	if(strlen(pattern) <=0){
		(void) printw("pattern is empty: %s", strerror(errno));
		(void) refresh();
		(void) sleep(2);
		srch_item[0].nr_entries=-1;
		LOG_CALL(1);
                return;
	}
	getmaxyx(searchwin, maxy, maxx);
	lcount:while((read=getline(&line, &len, fp)) != -1){
                DIR* d;
		char* l= malloc(strlen(line)+1);

                if(l == NULL)
                        goto lcount;
		if(len <=0){
			sfree(l);
			goto lcount;
		}
		memset(l, '\0', strlen(line)+1);
                strncpy(l, line, strlen(line)-1);
                d=opendir(l);
                if(d != NULL){
				closedir(d);
				sfree(l);
                                goto lcount;
                }
		 if(d != NULL)
                        closedir(d);

                else if(strstr((l+9), pattern) != NULL){ /* skip /proc/sys */
			if(biggest_strlen < strlen(l))
				biggest_strlen=strlen(l);
			i++;
			if(i >= maxy-2){ /*this is a limitation of the window size */
				srch_item[0].nr_entries =i-1;	/*display only what the screen can display */
				srch_item[0].entry_len = biggest_strlen;
				fclose(fp);
				sfree(l);
				LOG_CALL(1);
				return;	
			}

			memset(srch_item[i].fname, 0, strlen(l)+1);
                }
		sfree(l);

        }
	if(i ==0){
		srch_item[0].nr_entries = 0;
		LOG_CALL(1);
		return;
	}
	srch_item[0].nr_entries = i;
	srch_item[0].entry_len = biggest_strlen;
	
	if(fp != NULL)
		fclose(fp);
	LOG_CALL(1);
}
